/**
 * This is common js for intercept with Native Application
 * using jsbridge
 *
 * Version 1.0.0
 *
 *
 */
(function() {

	// 判断window对象中是否存在"J2C"，如果不存在，注册命名空间 'J2C' 到window对象上
	if(undefined == window['J2C']){
		window['J2C'] = {}

		connectWebViewJavascriptBridge(function(bridge) {
            bridge.init(function(message, responseCallback) {
                console.log('JS got a message', message);
                var data = {
                    'Javascript Responds': '测试中文!'
                };
                console.log('JS responding with', data);
                responseCallback(data);
            });

//            bridge.registerHandler("functionInJs", function(data, responseCallback) {
//                document.getElementById("show").innerHTML = ("data from Java: = " + data);
//                var responseData = "Javascript Says Right back aka!";
//                responseCallback(responseData);
//            });
        })

	}


	 function connectWebViewJavascriptBridge(callback) {
         if (window.WebViewJavascriptBridge) {
             callback(WebViewJavascriptBridge)
         } else {
             document.addEventListener(
                 'WebViewJavascriptBridgeReady'
                 , function() {
                     callback(WebViewJavascriptBridge)
                 },
                 false
             );
         }
     }

	// ---------- JS -> Native ---------------------

    /**
     * J2C环境Ready的Callback
     */
    j2cReadyCallback = function(event){
    	alert("J2C环境准备好了，喊你回家吃饭");
    };

    /*
    * 注册J2C环境Ready的Callback
    */
    function registerJ2CReadyCallback(callback){

    	if(window.webkit){
    		setTimeout(function(){
    			callback('j2c_ready');
    		},10);

    	}else if(window.WebViewJavascriptBridge){
    		setTimeout(function(){
    			callback('j2c_ready');
    		},10);

    	}else{
    		j2cReadyCallback = callback;
    		//把环境Ready的事件回调注册到document中
    	    var j2jdoc = document;
    	    j2jdoc.addEventListener('WebViewJavascriptBridgeReady', j2cReadyCallback , false);
    	}
	}


    /*
    * 通过原生弹出一个Toast
    * toast：显示内容
    */
    function showNativeToast(toast) {
		if(window.webkit){
			window.webkit.messageHandlers.showToast.postMessage({'body': toast});
		}else if(window.WebViewJavascriptBridge){
			window.WebViewJavascriptBridge.callHandler(
	                'Native.showToast'
	                , {'param': toast}
	                , function(responseData) {}
	            );
		}else{
			 throw new Error('J2J env init error');
		}
	}

    /*
    * 通过createNewWebPage打开一个新的page
    * obj内容：
    * pageId:界面标识
    * url:地址
    * param:当前界面向新界面传的的参数
    * titleName：导航栏标题
    * color:导航栏颜色
    */
	function createNewWebPage(obj) {
    		if(window.webkit){
    			 window.webkit.messageHandlers.createNewWebPage.postMessage({'body': obj});
    		}else if(window.WebViewJavascriptBridge){
    			window.WebViewJavascriptBridge.callHandler(
    	                'Native.createNewWebPage'
    	                , obj
    	                , function(responseData) {}
    	            );
    		}else{
    			 throw new Error('J2C env init error');
    		}
    }

    /*
    * 定义获取从其他页面createNewWebPage到自己时其他页面传入的参数的callback
    * obj既createNewPage中的param
    */
    var fetchParamCallback = new function(obj){};
    /*
    * 获取从其他页面createNewWebPage到自己时其他页面传入的参数
    */
    function fetchParam(callback) {
        fetchParamCallback = callback;
           if(window.webkit){
                window.webkit.messageHandlers.fetchParam.postMessage(1);
           }else if(window.WebViewJavascriptBridge){
               window.WebViewJavascriptBridge.callHandler(
                       'Native.fetchParam'
                       , {}
                       , function(responseData) {}
                   );
           }else{
               throw new Error('J2C env init error');
           }
    }

    /*
    * 获取从其他页面createNewWebPage到自己时其他页面传入的参数的Native回调
    */
    function onFetchParamSuccessed(obj){
            	fetchParamCallback(obj);
    }

    /**
     * 向其他页面发送消息
     * obj对象包含下列内容
     * （required）pageId:所在page的id，即发送这个请求本身这个page的id
     * （required）result：需要传递的内容
     * （opt）toPageId：传递到的page的id（可以没有这个字段），默认传给栈中上一个page
     */
    function sendResultToOtherPage(obj) {
    		if(window.webkit){
    			window.webkit.messageHandlers.sendResultToOtherPage.postMessage({'body': obj});
    		}else if(window.WebViewJavascriptBridge){
    			window.WebViewJavascriptBridge.callHandler(
    	                'Native.sendResultToOtherPage'
    	                , obj
    	                , function(responseData) {}
    	            );
    		}else{
    			 throw new Error('J2J env init error');
    		}
    }

    /*
    * 定义接收其他页面返回的receiver
    *（required）pageId:发送消息所在page的id
    * （required）result：需要传递的内容
    * （opt）toPageId：（可以没有这个字段），这里就是接收这个obj的page的id
    */
    var receiverCallback = new function(obj){};

    /*
    *注册其他页面返回的receiver
    */
    function registerReceiver(receiver) {
        receiverCallback = receiver;
    }

    /*
    * 触发Receiver的Native回调
    *（required）pageId:发送消息所在page的id
    * （required）result：需要传递的内容
    * （opt）toPageId：（可以没有这个字段），这里就是接收这个obj的page的id
    */
    function onReceiverResult(obj){
        receiverCallback(obj);
    }


   /**
    * 调用申请使用net request (不要直接使用，请使用封装函数sendRequest)
    * serialNum:序列号（目前可以暂时赋空值）
    * requestUrl:端口之后的地址
    * requestMethod：请求方法
    * requestQueries：请求参数（适用于get请求，问号后的请求参数封成的对象）
    * requestBody：请求体
    * callback：回调函数
    *  callback(statusObject, responseBody, responseHeaders);
    */
   function requestNet(serialNum, requestUrl, requestMethod, requestQueries, requestBody, requestHeaders) {
        if(window.webkit){
            window.webkit.messageHandlers.requestNet.postMessage({'serialNum':serialNum,'requestQueries':requestQueries, 'requestUrl':requestUrl, 'requestMethod':requestMethod, 'requestBody':requestBody, 'requestHeaders':requestHeaders});
        }else if(window.WebViewJavascriptBridge){
            window.WebViewJavascriptBridge.callHandler(
                        'Native.requestNet'
                        , {'serialNum': serialNum,
                            'requestUrl':requestUrl,
                            'requestMethod':requestMethod,
                            'requestQueries':requestQueries,
                            'requestHeaders':requestHeaders,
                            'requestBody':requestBody}
                        , function(responseData) {}
                    );
        }else{
            throw new Error('J2C env init error');
        }
   }

    /*
    * 存储js端请求回调的数组，用于后续根据serialNum确定回调函数的触发
    */
    var callArray = [];

    /**
     * Native中invoke了netRuest后回调JS的方法
     * serialNum：请求编号
     * statusObject：格式如下
     * {"code":200,"errormsg":"details"}
     * responseBody:服务端返回body体的内容，不做封装（如果确定服务端返回的json，可以按照json对象解析）
     * responseHeaders：服务端返回的header内容，json对象封装
     */
    function onRequestNetSuccessed(serialNum,statusObject,responseBody,responseHeaders){

//        alert("net back serialNum is:"+serialNum);
        for (x in callArray)
        {
            var s = callArray[x].serialNum;
            var netback = callArray[x].callback;

            if(s == serialNum){
                netback(statusObject, responseBody, responseHeaders);
                return;
            }
        }
    }


    /*
    * 网络请求封装函数，对netRequest进行封装，防止乱序
    * requestUrl:端口之后的地址
    * requestMethod：请求方法
    * requestQueries：请求参数（适用于get请求，问号后的请求参数封成的对象）
    * requestBody：请求体
    * callback：回调函数
    *  callback(statusObject, responseBody, responseHeaders);
    */
    function sendRequest(requestUrl, requestMethod, requestQueries, requestBody, requestHeaders , callback){

        var serialNum = uuid();

        var call = {};
        call.serialNum = serialNum;
        call.callback = callback;
        callArray.push(call);

        requestNet(serialNum, requestUrl, requestMethod, requestQueries, requestBody, requestHeaders );
    }





    /*
    * deprecated
    * 已过时(请使用buildTitleButtonGroup)
    *
    * 定义接收TitleBar的右侧Function按钮点击事件的callback
    *（required）functionId:TitleBar的右侧Function按钮点击事件的functionId
    * functionId的取值范围：
    * 首页我的图标 -> "ME"
    *
    * 其他的后续根据需要补充
    */
    var titleBarFunctionCallback = new function(obj){};

    /*
    * deprecated
    * 已过时(请使用buildTitleButtonGroup)
    *
    * 定义注册接收TitleBar的右侧Function按钮点击事件的callback的方法
    */
    function registerTitleBarFunctionCallback(callback) {
        titleBarFunctionCallback = callback;
    }

    /*
    * deprecated
    * 已过时(请使用buildTitleButtonGroup)
    *
    * 触发TitleBar的右侧Function按钮点击事件的Native回调
    *（required）functionId:TitleBar的右侧Function按钮点击事件的functionId
    */
    function onTitleBarFunctionInvoked(obj){
        titleBarFunctionCallback(obj);
    }


    //----------- others -----------------------------------------------

    /*
    * 产生uuid
    */
    function uuid() {
        var s = [];
        var hexDigits = "0123456789abcdef";
        for (var i = 0; i < 36; i++) {
            s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
        }
        s[14] = "4";  // bits 12-15 of the time_hi_and_version field to 0010
        s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);  // bits 6-7 of the clock_seq_hi_and_reserved to 01
        s[8] = s[13] = s[18] = s[23] = "-";

        var uuid = s.join("");
        return uuid;
    }

    /**
	 * 本地化存储函数
	 * @param {*} paramJsonObj
	 */
	function saveNativeStorage(paramJsonObj) {
		if(window.webkit) {
			window.webkit.messageHandlers.saveNativeStorage.postMessage({'body': paramJsonObj});

		} else if(window.WebViewJavascriptBridge) {

			window.WebViewJavascriptBridge.callHandler(
				'Native.saveNativeStorage', paramJsonObj,
				function(responseData) {}
			);

		} else {
			throw new Error('J2C env init error');
		}
	}


	var getNativeStorageCallback = new function(obj){};

	/**
	 * 获取本地化存储
	 * @param skey: 键
	 * @param invokeCallback: 回调
	 */
	function getNativeStorage(skey, invokeCallback) {
		if(invokeCallback != null)
        getNativeStorageCallback = invokeCallback;

		if(window.webkit) {
			window.webkit.messageHandlers.getNativeStorage.postMessage({'body': skey});
		} else if(window.WebViewJavascriptBridge) {
			window.WebViewJavascriptBridge.callHandler(
				'Native.getNativeStorage', skey,
				function(responseData) {}
			);
		} else {
			throw new Error('J2C env init error');
		}
	}

	/**
	 * 获取本地化存储的回调
	 */
	function onGetNativeStorageSuccessed (callbackparam) {
		getNativeStorageCallback(callbackparam);
	};




    var pickPhotosCallback = new function(obj){};

    /**
     * 选择图片拍照
     * maxNum:最大张数
     * invokeCallback:回调，参数为一个array，参数格式如下
     * [{"dis":"显示时使用的地址，在img标签的src中可以直接使用","use":"真实的文件地址，上传时传递给原生代码使用"},{},{}]
     */
    function pickPhotos(maxNum, invokeCallback) {
        if(invokeCallback != null)
        pickPhotosCallback = invokeCallback;

        if(window.webkit) {
            window.webkit.messageHandlers.pickPhotos.postMessage({'body': maxNum});
        } else if(window.WebViewJavascriptBridge) {
            window.WebViewJavascriptBridge.callHandler(
                'Native.pickPhotos', maxNum,
                function(responseData) {}
            );
        } else {
            throw new Error('J2C env init error');
        }
    }

    /**
     * 获取选择图片地址的回调
     */
    function onPickPhotosSuccessed (callbackparam) {
        pickPhotosCallback(callbackparam);
    };




    /**
	 * pdf下载及展示(仅支持get方式)
	 * @param {*} paramJsonObj
	 * url：pdf下载地址
	 * titleName：展示pdf页面标题
	 * requestHeaders: 请求头
	 * requestQueries：请求参数
	 */
	function viewPdf(paramJsonObj) {
		if(window.webkit) {
			window.webkit.messageHandlers.viewPdf.postMessage({'body': paramJsonObj});

		} else if(window.WebViewJavascriptBridge) {

			window.WebViewJavascriptBridge.callHandler(
				'Native.viewPdf', paramJsonObj,
				function(responseData) {}
			);

		} else {
			throw new Error('J2C env init error');
		}
	}


    /*
    * 回退到指定page
    * obj：
    * toPageId:回退到的指定page的Id，不能没有，可以等于一个空串，表示回退到上一个
    */
    function backToPage(obj) {
		if(window.webkit){
			window.webkit.messageHandlers.backToPage.postMessage({'body': obj});
		}else if(window.WebViewJavascriptBridge){
			window.WebViewJavascriptBridge.callHandler(
	                'Native.backToPage'
	                , obj
	                , function(responseData) {}
	            );
		}else{
			 throw new Error('J2J env init error');
		}
	}


    /*
    * 使用外部瀏覽器打開鏈接
    * obj：
    * url：打開的鏈接
    */
    function openInBrowser(obj) {
		if(window.webkit){
			window.webkit.messageHandlers.openInBrowser.postMessage({'body': obj});
		}else if(window.WebViewJavascriptBridge){
			window.WebViewJavascriptBridge.callHandler(
	                'Native.openInBrowser'
	                , obj
	                , function(responseData) {}
	            );
		}else{
			 throw new Error('J2J env init error');
		}
	}




    var scanFaceCallback = new function(obj){};

    /**
     * 活体识别
     * obj:调用参数
     * obj至少包含如下：
     * {"type": 使用的第三方（yiwei）},其他如有其他字段需要传递，在obj中进行字段扩展
     * invokeCallback:回调，参数为一个jsonobj，参数格式如下
     * { "result": "success"/"fail"其中success表示活体识别成功，fail表示失败, "type":"使用的第三方（yiwei）,"face":人脸base64String}, 其他如有其他字段需要传递，在对象中进行字段扩展
     */
    function scanFace(obj, invokeCallback) {
        if(invokeCallback != null)
        scanFaceCallback = invokeCallback;

        if(window.webkit) {
            window.webkit.messageHandlers.scanFace.postMessage({'body': obj});
        } else if(window.WebViewJavascriptBridge) {
            window.WebViewJavascriptBridge.callHandler(
                'Native.scanFace', obj,
                function(responseData) {}
            );
        } else {
            throw new Error('J2C env init error');
        }
    }

    /**
     * 获取活体的回调
     */
    function onScanFaceSuccessed (callbackparam) {
        scanFaceCallback(callbackparam);
    };




    var buildTitleButtonGroupCallback = new function(obj){};

    /**
     * 创建原生TitleBar上的按钮组
     * array:按钮数组 [obj,..]
     * obj为一个按钮的配置，至少包含如下：
     * {"type":"text/image","id":"ME/...用于回调时带回判断哪个按钮被点击","content":"如果是text那么content表示文字内容，如果是image那么content表示图片的内容"}
     * image类型（content）对照关系：
     * IMG_ME(我的图标)
     * invokeCallback:回调，参数为一个jsonobj，参数格式如下
     * { "id": "调用时给的id，点击哪个按钮就传相应的id"}
     */
    function buildTitleButtonGroup(array, invokeCallback) {
        if(invokeCallback != null)
        buildTitleButtonGroupCallback = invokeCallback;

        if(window.webkit) {
            window.webkit.messageHandlers.buildTitleButtonGroup.postMessage({'body': array});
        } else if(window.WebViewJavascriptBridge) {
            window.WebViewJavascriptBridge.callHandler(
                'Native.buildTitleButtonGroup', array,
                function(responseData) {}
            );
        } else {
            throw new Error('J2C env init error');
        }
    }

    /**
     * 获取TitleButton按钮点击的回调
     */
    function onBuildTitleButtonGroupSuccessed (callbackparam) {
        buildTitleButtonGroupCallback(callbackparam);
    };







    /*
    * 注册接收广播的callback
    * feature : 广播识别码
    * callback: function(obj){...}
    * obj:广播传递消息对象(json对象)
    *
    */
    function registerBroadcast(feature,callback) {

        var serialNum = uuid();
        var call = {};
        call.serialNum = serialNum;
        call.callback = callback;
        callArray.push(call);

        var realCall = {};
        realCall.feature = feature;
        realCall.serialNum = serialNum;

        if(window.webkit) {
            window.webkit.messageHandlers.registerBroadcast.postMessage({'body': realCall});

        } else if(window.WebViewJavascriptBridge) {

            window.WebViewJavascriptBridge.callHandler(
                'Native.registerBroadcast', realCall,
                function(responseData) {}
            );

        } else {
            throw new Error('J2C env init error');
        }

    }



    /**
     * 发送广播
     * feature：广播识别码
     * obj：广播传递消息对象(json对象)
     *
     */
    function sendBroadcast(feature,obj){

        var realCall = {};
        realCall.feature = feature;
        realCall.obj = obj;

        if(window.webkit) {
            window.webkit.messageHandlers.sendBroadcast.postMessage({'body': realCall});

        } else if(window.WebViewJavascriptBridge) {

            window.WebViewJavascriptBridge.callHandler(
                'Native.sendBroadcast', realCall,
                function(responseData) {}
            );

        } else {
            throw new Error('J2C env init error');
        }



    }





    /**
     * Native中invoke了onBroadcastInvoked后回调JS方法
     * serialNum：请求编号
     * obj：广播传递消息对象(json对象)
     *
     */
    function onBroadcastInvoked(serialNum,obj){

//        alert("net back serialNum is:"+serialNum);
        for (x in callArray)
        {
            var s = callArray[x].serialNum;
            var back = callArray[x].callback;

            if(s == serialNum){
                back(obj);
                return;
            }
        }
    }









	//---------------------deprecated --------------------------------------



	// 把函数注册到命名空间中
	window['J2C']['registerJ2CReadyCallback'] = registerJ2CReadyCallback;
	window['J2C']['createNewWebPage'] = createNewWebPage;
    window['J2C']['fetchParam'] = fetchParam;
    window['J2C']['onFetchParamSuccessed'] = onFetchParamSuccessed;
    window['J2C']['registerReceiver'] = registerReceiver;
    window['J2C']['onReceiverResult'] = onReceiverResult;
    window['J2C']['sendResultToOtherPage'] = sendResultToOtherPage;

    window['J2C']['sendRequest'] = sendRequest;
    window['J2C']['onRequestNetSuccessed'] = onRequestNetSuccessed;

    window['J2C']['registerTitleBarFunctionCallback'] = registerTitleBarFunctionCallback;
    window['J2C']['onTitleBarFunctionInvoked'] = onTitleBarFunctionInvoked;

    window['J2C']['showNativeToast'] = showNativeToast;
    window['J2C']['saveNativeStorage'] = saveNativeStorage;
    window['J2C']['getNativeStorage'] = getNativeStorage;
    window['J2C']['onGetNativeStorageSuccessed'] = onGetNativeStorageSuccessed;
    window['J2C']['pickPhotos'] = pickPhotos;
    window['J2C']['onPickPhotosSuccessed'] = onPickPhotosSuccessed;
    window['J2C']['viewPdf'] = viewPdf;
    window['J2C']['backToPage'] = backToPage;
    window['J2C']['openInBrowser'] = openInBrowser;
    window['J2C']['scanFace'] = scanFace;
    window['J2C']['onScanFaceSuccessed'] = onScanFaceSuccessed;

    window['J2C']['buildTitleButtonGroup'] = buildTitleButtonGroup;
    window['J2C']['onBuildTitleButtonGroupSuccessed'] = onBuildTitleButtonGroupSuccessed;


     window['J2C']['registerBroadcast'] = registerBroadcast;
     window['J2C']['onBroadcastInvoked'] = onBroadcastInvoked;
     window['J2C']['sendBroadcast'] = sendBroadcast;


})();


